@@ -1,22 +1,10 @@
-/* This file is part of the KDE libraries
+/*
+    This file is part of the KDE libraries
 
-   Copyright (c) 2000,2001 Dawit Alemayehu <adawit@kde.org>
-   Copyright (c) 2000,2001 Carsten Pfeiffer <pfeiffer@kde.org>
+    SPDX-FileCopyrightText: 2000, 2001 Dawit Alemayehu <adawit@kde.org>
+    SPDX-FileCopyrightText: 2000, 2001 Carsten Pfeiffer <pfeiffer@kde.org>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Lesser General Public
-   License (LGPL) as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Lesser General Public License for more details.
-
-   You should have received a copy of the GNU Lesser General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.1-or-later
 */
 
 #ifndef KCOMBOBOX_H
@@ -28,6 +16,7 @@
 #include <kcompletion_export.h>
 
 #include <QComboBox>
+#include <memory>
 
 class KCompletionBox;
 class KComboBoxPrivate;
@@ -36,6 +25,8 @@ class QLineEdit;
 class QMenu;
 
 /**
+ * @class KComboBox kcombobox.h KComboBox
+ *
  * @short A combo box with completion support.
  *
  * This widget inherits from QComboBox and implements the following
@@ -46,7 +37,7 @@ class QMenu;
  *   @li a popup menu item that can be used to allow the user to change
  * the text completion mode on the fly.
  *
- * To support these new features, KComboBox emits a few additional signals
+ * To support these additional features, KComboBox emits a few additional signals
  * such as completion(const QString&) and textRotation(KeyBindingType).
  * The completion signal can be connected to a slot that will assist the user in
  * filling out the remaining text while the rotation signal can be used to traverse
@@ -62,7 +53,7 @@ class QMenu;
  * internally whenever a completion object is created through either one of the
  * methods mentioned above. If you do not need this functionality, simply use
  * KCompletionBase::setHandleSignals(bool) or alternatively set the boolean
- * parameter in the @p setCompletionObject call to false.
+ * parameter in the @c setCompletionObject call to false.
  *
  * Beware: The completion object can be deleted on you, especially if a call
  * such as setEditable(false) is made. Store the pointer at your own risk,
@@ -75,7 +66,7 @@ class QMenu;
  * useGlobalSettings(). An alternate method would be to default individual
  * key bindings by using setKeyBinding() with the default second argument.
  *
- * A non-editable combo box only has one completion mode, @p CompletionAuto.
+ * A non-editable combo box only has one completion mode, @c CompletionAuto.
  * Unlike an editable combo box, the CompletionAuto mode works by matching
  * any typed key with the first letter of entries in the combo box. Please note
  * that if you call setEditable(false) to change an editable combo box to a
@@ -136,7 +127,7 @@ class QMenu;
  * combo->setKeyBinding(KCompletionBase::TextCompletion, Qt::End);
  * \endcode
  *
- * \image html kcombobox.png "KDE Combo Boxes, one non-editable, one editable with KUrlCompletion"
+ * \image html kcombobox.png "KComboBox widgets, one non-editable, one editable with KUrlCompletion"
  *
  * @author Dawit Alemayehu <adawit@kde.org>
  */
@@ -144,11 +135,10 @@ class KCOMPLETION_EXPORT KComboBox : public QComboBox, public KCompletionBase //
 {
     Q_OBJECT
     Q_PROPERTY(bool autoCompletion READ autoCompletion WRITE setAutoCompletion)
-#ifndef KCOMPLETION_NO_DEPRECATED
+#if KCOMPLETION_BUILD_DEPRECATED_SINCE(5, 0)
     Q_PROPERTY(bool urlDropsEnabled READ urlDropsEnabled WRITE setUrlDropsEnabled)
 #endif
     Q_PROPERTY(bool trapReturnKey READ trapReturnKey WRITE setTrapReturnKey)
-    Q_DECLARE_PRIVATE(KComboBox)
 
 public:
 
@@ -157,39 +147,43 @@ public:
     *
     * @param parent The parent object of this widget
     */
-    explicit KComboBox(QWidget *parent = 0);
+    explicit KComboBox(QWidget *parent = nullptr);
 
     /**
     * Constructs an editable or read-only combo box.
     *
-    * @param rw When @p true, widget will be editable.
+    * @param rw When @c true, widget will be editable.
     * @param parent The parent object of this widget.
     */
-    explicit KComboBox(bool rw, QWidget *parent = 0);
+    explicit KComboBox(bool rw, QWidget *parent = nullptr);
 
     /**
     * Destructor.
     */
-    virtual ~KComboBox();
+    ~KComboBox() override;
 
+#if KCOMPLETION_ENABLE_DEPRECATED_SINCE(4, 5)
     /**
      * Deprecated to reflect Qt api changes
      * @deprecated since 4.5
      */
-#ifndef KCOMPLETION_NO_DEPRECATED
-    KCOMPLETION_DEPRECATED void insertURL(const QUrl &url, int index = -1)
+    KCOMPLETION_DEPRECATED_VERSION(4, 5, "Use KComboBox::insertUrl(int, const QUrl&)")
+    void insertURL(const QUrl &url, int index = -1)
     {
         insertUrl(index < 0 ? count() : index, url);
     }
-    KCOMPLETION_DEPRECATED void insertURL(const QPixmap &pixmap, const QUrl &url, int index = -1)
+    KCOMPLETION_DEPRECATED_VERSION(4, 5, "Use KComboBox::insertUrl(int, const QIcon&, const QUrl&)")
+    void insertURL(const QPixmap &pixmap, const QUrl &url, int index = -1)
     {
         insertUrl(index < 0 ? count() : index, QIcon(pixmap), url);
     }
-    KCOMPLETION_DEPRECATED void changeURL(const QUrl &url, int index)
+    KCOMPLETION_DEPRECATED_VERSION(4, 5, "Use KComboBox::changeUrl(int, const QUrl&)")
+    void changeURL(const QUrl &url, int index)
     {
         changeUrl(index, url);
     }
-    KCOMPLETION_DEPRECATED void changeURL(const QPixmap &pixmap, const QUrl &url, int index)
+    KCOMPLETION_DEPRECATED_VERSION(4, 5, "Use KComboBox::changeUrl(int, const QIcon&, const QUrl&)")
+    void changeURL(const QPixmap &pixmap, const QUrl &url, int index)
     {
         changeUrl(index, QIcon(pixmap), url);
     }
@@ -265,7 +259,7 @@ public:
     /**
     * Reimplemented from QComboBox.
     *
-    * If @p true, the completion mode will be set to automatic.
+    * If @c true, the completion mode will be set to automatic.
     * Otherwise, it is defaulted to the global setting. This
     * method has been replaced by the more comprehensive
     * setCompletionMode().
@@ -277,14 +271,15 @@ public:
     /**
     * Reimplemented from QComboBox.
     *
-    * Returns @p true if the current completion mode is set
+    * Returns @c true if the current completion mode is set
     * to automatic. See its more comprehensive replacement
     * completionMode().
     *
-    * @return @p true when completion mode is automatic.
+    * @return @c true when completion mode is automatic.
     */
     bool autoCompletion() const;
 
+#if KCOMPLETION_BUILD_DEPRECATED_SINCE(4, 5)
     /**
     * Enables or disables the popup (context) menu.
     *
@@ -296,13 +291,14 @@ public:
     * Call this function with the argument set to false to disable the popup
     * menu.
     *
-    * @param showMenu If @p true, show the context menu.
+    * @param showMenu If @c true, show the context menu.
     * @deprecated since 4.5, use setContextMenuPolicy instead
     */
-#ifndef KCOMPLETION_NO_DEPRECATED
-    virtual KCOMPLETION_DEPRECATED void setContextMenuEnabled(bool showMenu);
+    KCOMPLETION_DEPRECATED_VERSION(4, 5, "Use QWidget::setContextMenuPolicy(...)")
+    virtual void setContextMenuEnabled(bool showMenu);
 #endif
 
+#if KCOMPLETION_ENABLE_DEPRECATED_SINCE(5, 0)
     /**
      * Enables/Disables handling of URL drops.
      *
@@ -310,15 +306,15 @@ public:
      * be inserted. Otherwise the default behavior of QComboBox is used,
      * which inserts the encoded URL.
      *
-     * @param enable If @p true, insert decoded URLs
+     * @param enable If @c true, insert decoded URLs
      * @deprecated since 5.0. Use lineEdit()->installEventFilter with a LineEditUrlDropEventFilter
      */
-#ifndef KCOMPLETION_NO_DEPRECATED
-    KCOMPLETION_DEPRECATED void setUrlDropsEnabled(bool enable);
+    KCOMPLETION_DEPRECATED_VERSION(5, 0, "Use KComboBox::lineEdit()->installEventFilter(...) with a LineEditUrlDropEventFilter")
+    void setUrlDropsEnabled(bool enable);
 #endif
 
     /**
-     * Returns @p true when decoded URL drops are enabled
+     * Returns @c true when decoded URL drops are enabled
      */
     bool urlDropsEnabled() const;
 
@@ -326,10 +322,10 @@ public:
      * Convenience method which iterates over all items and checks if
      * any of them is equal to @p text.
      *
-     * If @p text is an empty string, @p false
+     * If @p text is an empty string, @c false
      * is returned.
      *
-     * @return @p true if an item with the string @p text is in the combo box.
+     * @return @c true if an item with the string @p text is in the combo box.
      */
     bool contains(const QString &text) const;
 
@@ -349,22 +345,21 @@ public:
     void setTrapReturnKey(bool trap);
 
     /**
-     * @return @p true if key events of Key_Return or Key_Enter will
-     * be stopped; @p false if they will be propagated.
+     * @return @c true if key events of Key_Return or Key_Enter will
+     * be stopped; @c false if they will be propagated.
      *
      * @see setTrapReturnKey ()
      */
     bool trapReturnKey() const;
 
     /**
-     * @returns the completion box that is used in completion mode
-     * CompletionPopup and CompletionPopupAuto.
-     *
      * This method will create a completion box by calling
      * KLineEdit::completionBox, if none is there yet.
      *
      * @param create Set this to false if you don't want the box to be created
      *               i.e. to test if it is available.
+     * @returns the completion box that is used in completion mode
+     * CompletionPopup and CompletionPopupAuto.
      */
     KCompletionBox *completionBox(bool create = true);
 
@@ -388,6 +383,14 @@ public:
      */
     void setEditable(bool editable);
 
+    /**
+     * Pointer to KLineEdit's context menu, or nullptr if it does not exist at
+     * the given moment.
+     *
+     * @since 5.78
+     */
+    QMenu *contextMenu() const;
+
 Q_SIGNALS:
     /**
     * Emitted when the user presses the Enter key.
@@ -413,7 +416,7 @@ Q_SIGNALS:
     * The argument is the current text being edited.
     *
     * Note that this signal is @em not available when the widget is non-editable
-    * or the completion mode is set to @p CompletionNone.
+    * or the completion mode is set to @c CompletionNone.
     */
     void completion(const QString &);
 
@@ -426,8 +429,8 @@ Q_SIGNALS:
      * Emitted when the text rotation key bindings are pressed.
      *
      * The argument indicates which key binding was pressed. In this case this
-     * can be either one of four values: @p PrevCompletionMatch,
-     * @p NextCompletionMatch, @p RotateUp or @p RotateDown.
+     * can be either one of four values: @c PrevCompletionMatch,
+     * @c NextCompletionMatch, @c RotateUp or @c RotateDown.
      *
      * Note that this signal is @em not emitted if the completion
      * mode is set to CompletionNone.
@@ -466,7 +469,7 @@ public Q_SLOTS:
     * iteration will not work if there are no previous matches, i.e.
     * no text has been completed and the *nix shell history list
     * rotation is only available if the insertion policy for this
-    * widget is set either @p QComobBox::AtTop or @p QComboBox::AtBottom.
+    * widget is set either @c QComobBox::AtTop or @c QComboBox::AtBottom.
     * For other insertion modes whatever has been typed by the user
     * when the rotation event was initiated will be lost.
     *
@@ -480,13 +483,13 @@ public Q_SLOTS:
      * This function is an implementation for
      * KCompletionBase::setCompletedText.
      */
-    void setCompletedText(const QString &) Q_DECL_OVERRIDE;
+    void setCompletedText(const QString &) override;
 
     /**
      * Sets @p items into the completion box if completionMode() is
      * CompletionPopup. The popup will be shown immediately.
      */
-    void setCompletedItems(const QStringList &items, bool autosubject = true) Q_DECL_OVERRIDE;
+    void setCompletedItems(const QStringList &items, bool autosubject = true) override;
 
     /**
      * Selects the first item that matches @p item.
@@ -501,8 +504,8 @@ protected Q_SLOTS:
     /**
     * Completes text according to the completion mode.
     *
-    * Note: this method is not invoked if the completion mode is
-    * set to @p CompletionNone. Also if the mode is set to @p CompletionShell
+    * @note This method is not invoked if the completion mode is
+    * set to @c CompletionNone. Also if the mode is set to @c CompletionShell
     * and multiple matches are found, this method will complete the
     * text to the first match with a beep to indicate that there are
     * more matches. Then any successive completion key event iterates
@@ -522,10 +525,19 @@ protected:
     */
     virtual void setCompletedText(const QString &text, bool marked);
 
-    QSize minimumSizeHint() const Q_DECL_OVERRIDE;
+    // TODO KF6: make public like in base classes, so consumers do not need to cast to base classes
+    // when they have a KComboBox (or subclasses) object and want to access this property
+    QSize minimumSizeHint() const override;
+
+protected:
+    KComboBox(KComboBoxPrivate &dd, QWidget *parent);
 
 private:
-    const QScopedPointer<KComboBoxPrivate> d_ptr;
+    friend class KHistoryComboBox;
+    Q_DECLARE_PRIVATE(KComboBox)
+    std::unique_ptr<KComboBoxPrivate> const d_ptr;
+    // KF6 TODO: change private d_ptr to protected d_ptr, remove friend
+
     Q_PRIVATE_SLOT(d_func(), void _k_lineEditDeleted())
 };
 
